<?php

abstract class CM_Model_ActionLimit_Abstract extends CM_Model_Abstract {

    /**
     * @var array
     */
    public $_data;

    /**
     * @param int $actionType
     * @param int $actionVerb
     */
    public function __construct($actionType, $actionVerb) {
        parent::_construct(array('actionType' => (int) $actionType, 'actionVerb' => (int) $actionVerb));
    }

    /**
     * @param CM_Action_Abstract $action
     * @param int|null           $role
     * @param boolean            $first
     */
    abstract public function overshoot(CM_Action_Abstract $action, $role, $first);

    /**
     * @param CM_Action_Abstract $action
     * @return boolean
     */
    abstract public function getOvershootAllowed(CM_Action_Abstract $action);

    /**
     * @return int
     */
    public function getActionType() {
        return (int) $this->_getIdKey('actionType');
    }

    /**
     * @return int
     */
    public function getActionVerb() {
        return (int) $this->_getIdKey('actionVerb');
    }

    /**
     * @param int $role OPTIONAL
     *
     * @return int|null
     */
    public function getLimit($role = null) {
        $roles = $this->_get('roles');
        if (!isset($roles[$role])) {
            if ($role === null) {
                return null;
            }
            return $this->getLimit();
        }
        $limit = $roles[$role]['limit'];
        if (null !== $limit) {
            $limit = (int) $limit;
        }
        return $limit;
    }

    /**
     * @param int|null $role
     * @param int|null $limit
     */
    public function setLimit($role, $limit) {
        $role = $role ? (int) $role : null;
        $limit = !isset($limit) ? null : (int) $limit;
        if (CM_Db_Db::count('cm_actionLimit', array(
            'actionType' => $this->getActionType(),
            'actionVerb' => $this->getActionVerb(),
            'type'       => $this->getType(),
            'role'       => $role,
        ))
        ) {
            CM_Db_Db::update('cm_actionLimit', array('limit' => $limit), array(
                'actionType' => $this->getActionType(),
                'actionVerb' => $this->getActionVerb(),
                'type'       => $this->getType(),
                'role'       => $role,
            ));
        } else {
            CM_Db_Db::insert('cm_actionLimit', array(
                'limit'      => $limit,
                'actionType' => $this->getActionType(),
                'actionVerb' => $this->getActionVerb(),
                'type'       => $this->getType(),
                'role'       => $role,
            ));
        }
        $this->_change();
    }

    /**
     * @param int $role OPTIONAL
     *
     * @return int|null
     */
    public function getPeriod($role = null) {
        $roles = $this->_get('roles');
        if (!isset($roles[$role])) {
            if ($role === null) {
                return null;
            }
            return $this->getPeriod();
        }
        return (int) $roles[$role]['period'];
    }

    /**
     * @param int|null $role
     * @param int      $period
     */
    public function setPeriod($role, $period) {
        $role = $role ? (int) $role : null;
        $period = (int) $period;
        if (CM_Db_Db::count('cm_actionLimit', array(
            'actionType' => $this->getActionType(),
            'actionVerb' => $this->getActionVerb(),
            'type'       => $this->getType(),
            'role'       => $role,
        ))
        ) {
            CM_Db_Db::update('cm_actionLimit', array('period' => $period), array(
                'actionType' => $this->getActionType(),
                'actionVerb' => $this->getActionVerb(),
                'type'       => $this->getType(),
                'role'       => $role,
            ));
        } else {
            CM_Db_Db::insert('cm_actionLimit', array(
                'period'     => $period,
                'actionType' => $this->getActionType(),
                'actionVerb' => $this->getActionVerb(),
                'type'       => $this->getType(),
                'role'       => $role,
            ));
        }
        $this->_change();
    }

    /**
     * @param null|int $role
     * @return boolean
     */
    public function hasLimit($role = null) {
        $role = $role ? (int) $role : null;
        $roles = $this->_get('roles');
        return isset($roles[$role]);
    }

    /**
     * @param int|null $role
     */
    public function unsetLimit($role = null) {
        $role = $role ? (int) $role : null;
        CM_Db_Db::delete('cm_actionLimit', array(
            'actionType' => $this->getActionType(),
            'actionVerb' => $this->getActionVerb(),
            'type'       => $this->getType(),
            'role'       => $role,
        ));
        $this->_change();
    }

    /**
     * @param int|null $role
     * @param int|null $limit
     * @param int      $period
     * @return $this
     */
    public function setLimitAndPeriod($role, $limit, $period) {
        $this->setLimit($role, $limit);
        $this->setPeriod($role, $period);
        return $this;
    }

    public function jsonSerialize() {
        return [
            'actionType' => $this->getActionType(),
            'actionVerb' => $this->getActionVerb(),
        ];
    }

    protected function _loadData() {
        return array('roles' => CM_Db_Db::select('cm_actionLimit', array('role', 'limit', 'period'), array(
            'actionType' => $this->getActionType(),
            'actionVerb' => $this->getActionVerb(),
            'type'       => $this->getType(),
        ))->fetchAllTree(),);
    }

    /**
     * @param int $type
     * @param int $actionType
     * @param int $actionVerb
     * @return CM_Model_ActionLimit_Abstract
     */
    public static function factory($type, $actionType, $actionVerb) {
        $class = self::_getClassName($type);
        return new $class($actionType, $actionVerb);
    }

    /**
     * @param int $type OPTIONAL
     * @return CM_Paging_ActionLimit_All
     */
    public static function getAll($type = null) {
        if (!$type) {
            $type = static::getTypeStatic();
        }
        return new CM_Paging_ActionLimit_All($type);
    }
}
